@@ -123,16 +123,17 @@ typedef void (*LDAPOpHandler)(LDAPOp *op, LDAPMessage *res);
 typedef void (*LDAPOpDtor)(LDAPOp *op);
 
 struct LDAPOp {
-	LDAPOpHandler handler;
-	LDAPOpDtor    dtor;
-	EBookBackend    *backend;
-	EDataBook       *book;
-	EDataBookView   *view;
-	int            id;
+	LDAPOpHandler  handler;
+	LDAPOpDtor     dtor;
+	EBookBackend  *backend;
+	EDataBook     *book;
+	EDataBookView *view;
+	guint32        opid; /* the libebook operation id */
+	int            id;   /* the ldap msg id */
 };
 
 static void     ldap_op_add (LDAPOp *op, EBookBackend *backend, EDataBook *book,
-			     EDataBookView *view, int id, LDAPOpHandler handler, LDAPOpDtor dtor);
+			     EDataBookView *view, int opid, int msgid, LDAPOpHandler handler, LDAPOpDtor dtor);
 static void     ldap_op_finished (LDAPOp *op);
 
 static gboolean poll_ldap (EBookBackendLDAP *bl);
@@ -140,30 +141,44 @@ static gboolean poll_ldap (EBookBackendLDAP *bl);
 static EContact *build_contact_from_entry (LDAP *ldap, LDAPMessage *e, GList **existing_objectclasses);
 
 static void email_populate (EContact *contact, char **values);
-struct berval** email_ber (EContact *contact);
+static struct berval** email_ber (EContact *contact);
 static gboolean email_compare (EContact *contact1, EContact *contact2);
 
 static void homephone_populate (EContact *contact, char **values);
-struct berval** homephone_ber (EContact *contact);
+static struct berval** homephone_ber (EContact *contact);
 static gboolean homephone_compare (EContact *contact1, EContact *contact2);
 
 static void business_populate (EContact *contact, char **values);
-struct berval** business_ber (EContact *contact);
+static struct berval** business_ber (EContact *contact);
 static gboolean business_compare (EContact *contact1, EContact *contact2);
 
 static void anniversary_populate (EContact *contact, char **values);
-struct berval** anniversary_ber (EContact *contact);
+static struct berval** anniversary_ber (EContact *contact);
 static gboolean anniversary_compare (EContact *contact1, EContact *contact2);
 
 static void birthday_populate (EContact *contact, char **values);
-struct berval** birthday_ber (EContact *contact);
+static struct berval** birthday_ber (EContact *contact);
 static gboolean birthday_compare (EContact *contact1, EContact *contact2);
 
 static void category_populate (EContact *contact, char **values);
-struct berval** category_ber (EContact *contact);
+static struct berval** category_ber (EContact *contact);
 static gboolean category_compare (EContact *contact1, EContact *contact2);
 
+static void home_address_populate(EContact * card, char **values);
+static struct berval **home_address_ber(EContact * card);
+static gboolean home_address_compare(EContact * ecard1, EContact * ecard2);
+
+static void work_address_populate(EContact * card, char **values);
+static struct berval **work_address_ber(EContact * card);
+static gboolean work_address_compare(EContact * ecard1, EContact * ecard2);
+
+static void other_address_populate(EContact * card, char **values);
+static struct berval **other_address_ber(EContact * card);
+static gboolean other_address_compare(EContact * ecard1, EContact * ecard2);
+
 static void photo_populate (EContact *contact, struct berval **ber_values);
+static struct berval **photo_ber (EContact * contact);
+static gboolean photo_compare(EContact * ecard1, EContact * ecard2);
 
 static void cert_populate (EContact *contact, struct berval **ber_values);
 
@@ -235,12 +250,12 @@ struct prop_info {
 	E_STRING_PROP (E_CONTACT_ASSISTANT, "assistantName"), 
 
 	/* addresses */
-	STRING_PROP   (E_CONTACT_ADDRESS_LABEL_WORK,  "postalAddress"),
-	STRING_PROP   (E_CONTACT_ADDRESS_LABEL_HOME,  "homePostalAddress"),
-	E_STRING_PROP (E_CONTACT_ADDRESS_LABEL_OTHER, "otherPostalAddress"),
+	COMPLEX_PROP  (E_CONTACT_ADDRESS_LABEL_WORK, "postalAddress", work_address_populate, work_address_ber, work_address_compare),
+	COMPLEX_PROP  (E_CONTACT_ADDRESS_LABEL_HOME,     "homePostalAddress", home_address_populate, home_address_ber, home_address_compare),
+	E_COMPLEX_PROP(E_CONTACT_ADDRESS_LABEL_OTHER,    "otherPostalAddress", other_address_populate, other_address_ber, other_address_compare),
 
 	/* photos */
-	BINARY_PROP  (E_CONTACT_PHOTO,       "jpegPhoto", photo_populate, NULL/*XXX*/, NULL/*XXX*/),
+	BINARY_PROP  (E_CONTACT_PHOTO,       "jpegPhoto", photo_populate, photo_ber, photo_compare),
 
 	/* certificate foo. */
 	BINARY_PROP  (E_CONTACT_X509_CERT,   "userCertificate", cert_populate, NULL/*XXX*/, NULL/*XXX*/),
@@ -389,6 +404,7 @@ add_to_supported_fields (EBookBackendLDAP *bl, char **attrs, GHashTable *attr_ha
 			if (!strcmp (query_prop, "email")) {
 				bl->priv->supported_fields = g_list_append (bl->priv->supported_fields, g_strdup ("email_2"));
 				bl->priv->supported_fields = g_list_append (bl->priv->supported_fields, g_strdup ("email_3"));
+				bl->priv->supported_fields = g_list_append (bl->priv->supported_fields, g_strdup ("email_4"));
 			}
 			else if (!strcmp (query_prop, "business_phone")) {
 				bl->priv->supported_fields = g_list_append (bl->priv->supported_fields, g_strdup ("business_phone_2"));
@@ -396,6 +412,9 @@ add_to_supported_fields (EBookBackendLDAP *bl, char **attrs, GHashTable *attr_ha
 			else if (!strcmp (query_prop, "home_phone")) {
 				bl->priv->supported_fields = g_list_append (bl->priv->supported_fields, g_strdup ("home_phone_2"));
 			}
+			else if (!strcmp (query_prop, "category_list")) {
+				bl->priv->supported_fields = g_list_append (bl->priv->supported_fields, g_strdup ("categories"));
+			}
 		}
 	}
 }
@@ -639,7 +658,7 @@ e_book_backend_ldap_connect (EBookBackendLDAP *bl)
 
 	/* close connection first if it's open first */
 	if (blpriv->ldap)
-		ldap_unbind_ext (blpriv->ldap, NULL, NULL);
+		ldap_unbind_ext_s (blpriv->ldap, NULL, NULL);
 
 	blpriv->ldap = ldap_init (blpriv->ldap_host, blpriv->ldap_port);
 #if defined (DEBUG) && defined (LDAP_OPT_DEBUG_LEVEL)
@@ -652,20 +671,20 @@ e_book_backend_ldap_connect (EBookBackendLDAP *bl)
 	if (NULL != blpriv->ldap) {
 		int ldap_error;
 
+		ldap_error = ldap_set_option (blpriv->ldap, LDAP_OPT_PROTOCOL_VERSION, &protocol_version);
+		if (LDAP_OPT_SUCCESS != ldap_error) {
+			g_warning ("failed to set protocol version to LDAPv3");
+			bl->priv->ldap_v3 = FALSE;
+		}
+		else
+			bl->priv->ldap_v3 = TRUE;
+
 		if (bl->priv->use_tls != E_BOOK_BACKEND_LDAP_TLS_NO) {
 			int tls_level;
 
-			ldap_error = ldap_set_option (blpriv->ldap, LDAP_OPT_PROTOCOL_VERSION, &protocol_version);
-			if (LDAP_OPT_SUCCESS != ldap_error) {
-				g_warning ("failed to set protocol version to LDAPv3");
-				bl->priv->ldap_v3 = FALSE;
-			}
-			else
-				bl->priv->ldap_v3 = TRUE;
-
 			if (!bl->priv->ldap_v3 && bl->priv->use_tls == E_BOOK_BACKEND_LDAP_TLS_ALWAYS) {
 				g_message ("TLS not available (fatal version), v3 protocol could not be established (ldap_error 0x%02x)", ldap_error);
-				ldap_unbind (blpriv->ldap);
+				ldap_unbind_ext_s (blpriv->ldap, NULL, NULL);
 				blpriv->ldap = NULL;
 				return GNOME_Evolution_Addressbook_TLSNotAvailable;
 			}
@@ -679,7 +698,7 @@ e_book_backend_ldap_connect (EBookBackendLDAP *bl)
 				if (LDAP_SUCCESS != ldap_error) {
 					if (bl->priv->use_tls == E_BOOK_BACKEND_LDAP_TLS_ALWAYS) {
 						g_message ("TLS not available (fatal version), (ldap_error 0x%02x)", ldap_error);
-						ldap_unbind (blpriv->ldap);
+						ldap_unbind_ext_s (blpriv->ldap, NULL, NULL);
 						blpriv->ldap = NULL;
 						return GNOME_Evolution_Addressbook_TLSNotAvailable;
 					}
@@ -789,7 +808,8 @@ e_book_backend_ldap_reconnect (EBookBackendLDAP *bl, EDataBookView *book_view, i
 static void
 ldap_op_add (LDAPOp *op, EBookBackend *backend,
 	     EDataBook *book, EDataBookView *view,
-	     int id,
+	     int opid,
+	     int msgid,
 	     LDAPOpHandler handler, LDAPOpDtor dtor)
 {
 	EBookBackendLDAP *bl = E_BOOK_BACKEND_LDAP (backend);
@@ -797,7 +817,8 @@ ldap_op_add (LDAPOp *op, EBookBackend *backend,
 	op->backend = backend;
 	op->book = book;
 	op->view = view;
-	op->id = id;
+	op->opid = opid;
+	op->id = msgid;
 	op->handler = handler;
 	op->dtor = dtor;
 
@@ -931,13 +952,13 @@ free_mods (GPtrArray *mods)
 		int j;
 		g_free (mod->mod_type);
 
-		if (mod->mod_op & LDAP_MOD_BVALUES) {
+		if (mod->mod_op & LDAP_MOD_BVALUES && mod->mod_bvalues) {
 			for (j = 0; mod->mod_bvalues[j]; j++) {
 				g_free (mod->mod_bvalues[j]->bv_val);
 				g_free (mod->mod_bvalues[j]);
 			}
 		}
-		else {
+		else if (mod->mod_values) {
 			for (j = 0; mod->mod_values[j]; j++)
 				g_free (mod->mod_values[j]);
 		}
@@ -1155,8 +1176,9 @@ create_contact_handler (LDAPOp *op, LDAPMessage *res)
 	if (LDAP_RES_ADD != ldap_msgtype (res)) {
 		g_warning ("incorrect msg type %d passed to create_contact_handler", ldap_msgtype (res));
 		e_data_book_respond_create (op->book,
-					 GNOME_Evolution_Addressbook_OtherError,
-					 NULL);
+					    op->opid,
+					    GNOME_Evolution_Addressbook_OtherError,
+					    NULL);
 		ldap_op_finished (op);
 		return;
 	}
@@ -1173,6 +1195,7 @@ create_contact_handler (LDAPOp *op, LDAPMessage *res)
 	/* and lastly respond */
 	response = ldap_error_to_response (ldap_error);
 	e_data_book_respond_create (op->book,
+				    op->opid,
 				    response,
 				    create_op->new_contact);
 
@@ -1192,6 +1215,7 @@ create_contact_dtor (LDAPOp *op)
 static void
 e_book_backend_ldap_create_contact (EBookBackend *backend,
 				    EDataBook    *book,
+				    guint32       opid,
 				    const char   *vcard)
 {
 	LDAPCreateOp *create_op = g_new (LDAPCreateOp, 1);
@@ -1293,15 +1317,16 @@ e_book_backend_ldap_create_contact (EBookBackend *backend,
 	if (LDAP_SUCCESS != err) {
 		response = ldap_error_to_response (err);
 		e_data_book_respond_create (create_op->op.book,
-					 response,
-					 NULL);
+					    opid,
+					    response,
+					    NULL);
 		create_contact_dtor ((LDAPOp*)create_op);
 		return;
 	}
 	else {
 		g_print ("ldap_add_ext returned %d\n", err);
 		ldap_op_add ((LDAPOp*)create_op, backend, book,
-			     book_view, create_contact_msgid,
+			     book_view, opid, create_contact_msgid,
 			     create_contact_handler, create_contact_dtor);
 	}
 }
@@ -1324,8 +1349,9 @@ remove_contact_handler (LDAPOp *op, LDAPMessage *res)
 	if (LDAP_RES_DELETE != ldap_msgtype (res)) {
 		g_warning ("incorrect msg type %d passed to remove_contact_handler", ldap_msgtype (res));
 		e_data_book_respond_remove_contacts (op->book,
-						  GNOME_Evolution_Addressbook_OtherError,
-						  NULL);
+						     op->opid,
+						     GNOME_Evolution_Addressbook_OtherError,
+						     NULL);
 		ldap_op_finished (op);
 		return;
 	}
@@ -1341,8 +1367,9 @@ remove_contact_handler (LDAPOp *op, LDAPMessage *res)
 
 	ids = g_list_append (ids, remove_op->id);
 	e_data_book_respond_remove_contacts (remove_op->op.book,
-					  ldap_error_to_response (ldap_error),
-					  ids);
+					     op->opid,
+					     ldap_error_to_response (ldap_error),
+					     ids);
 	g_list_free (ids);
 }
 
@@ -1358,6 +1385,7 @@ remove_contact_dtor (LDAPOp *op)
 static void
 e_book_backend_ldap_remove_contacts (EBookBackend *backend,
 				     EDataBook    *book,
+				     guint32       opid,
 				     GList        *ids)
 {
 	LDAPRemoveOp *remove_op = g_new (LDAPRemoveOp, 1);
@@ -1385,15 +1413,16 @@ e_book_backend_ldap_remove_contacts (EBookBackend *backend,
 
 	if (ldap_error != LDAP_SUCCESS) {
 		e_data_book_respond_remove_contacts (remove_op->op.book,
-						  ldap_error_to_response (ldap_error),
-						  NULL);
+						     opid,
+						     ldap_error_to_response (ldap_error),
+						     NULL);
 		remove_contact_dtor ((LDAPOp*)remove_op);
 		return;
 	}
 	else {
 		g_print ("ldap_delete_ext returned %d\n", ldap_error);
 		ldap_op_add ((LDAPOp*)remove_op, backend, book,
-			     book_view, remove_msgid,
+			     book_view, opid, remove_msgid,
 			     remove_contact_handler, remove_contact_dtor);
 	}
 }
@@ -1430,8 +1459,9 @@ modify_contact_modify_handler (LDAPOp *op, LDAPMessage *res)
 	if (LDAP_RES_MODIFY != ldap_msgtype (res)) {
 		g_warning ("incorrect msg type %d passed to modify_contact_handler", ldap_msgtype (res));
 		e_data_book_respond_modify (op->book,
-					 GNOME_Evolution_Addressbook_OtherError,
-					 NULL);
+					    op->opid,
+					    GNOME_Evolution_Addressbook_OtherError,
+					    NULL);
 		ldap_op_finished (op);
 		return;
 	}
@@ -1447,8 +1477,9 @@ modify_contact_modify_handler (LDAPOp *op, LDAPMessage *res)
 
 	/* and lastly respond */
 	e_data_book_respond_modify (op->book,
-				 ldap_error_to_response (ldap_error),
-				 modify_op->contact);
+				    op->opid,
+				    ldap_error_to_response (ldap_error),
+				    modify_op->contact);
 	ldap_op_finished (op);
 }
 
@@ -1471,8 +1502,9 @@ modify_contact_search_handler (LDAPOp *op, LDAPMessage *res)
 		if (!e) {
 			g_warning ("uh, this shouldn't happen");
 			e_data_book_respond_modify (op->book,
-						 GNOME_Evolution_Addressbook_OtherError,
-						 NULL);
+						    op->opid,
+						    GNOME_Evolution_Addressbook_OtherError,
+						    NULL);
 			ldap_op_finished (op);
 			return;
 		}
@@ -1503,8 +1535,9 @@ modify_contact_search_handler (LDAPOp *op, LDAPMessage *res)
 		if (ldap_error != LDAP_SUCCESS) {
 			/* more here i'm sure */
 			e_data_book_respond_modify (op->book,
-						 ldap_error_to_response (ldap_error),
-						 NULL);
+						    op->opid,
+						    ldap_error_to_response (ldap_error),
+						    NULL);
 			ldap_op_finished (op);
 			return;
 		}
@@ -1538,8 +1571,9 @@ modify_contact_search_handler (LDAPOp *op, LDAPMessage *res)
 			else {
 				g_warning ("ldap_modify_ext returned %d\n", ldap_error);
 				e_data_book_respond_modify (op->book,
-							 ldap_error_to_response (ldap_error),
-							 NULL);
+							    op->opid,
+							    ldap_error_to_response (ldap_error),
+							    NULL);
 				ldap_op_finished (op);
 				return;
 			}
@@ -1551,8 +1585,9 @@ modify_contact_search_handler (LDAPOp *op, LDAPMessage *res)
 	else {
 		g_warning ("unhandled result type %d returned", msg_type);
 		e_data_book_respond_modify (op->book,
-					 GNOME_Evolution_Addressbook_OtherError,
-					 NULL);
+					    op->opid,
+					    GNOME_Evolution_Addressbook_OtherError,
+					    NULL);
 		ldap_op_finished (op);
 	}
 }
@@ -1574,6 +1609,7 @@ modify_contact_dtor (LDAPOp *op)
 static void
 e_book_backend_ldap_modify_contact (EBookBackend *backend,
 				    EDataBook    *book,
+				    guint32       opid,
 				    const char   *vcard)
 {
 	LDAPModifyOp *modify_op = g_new0 (LDAPModifyOp, 1);
@@ -1604,14 +1640,15 @@ e_book_backend_ldap_modify_contact (EBookBackend *backend,
 
 	if (ldap_error == LDAP_SUCCESS) {
 		ldap_op_add ((LDAPOp*)modify_op, backend, book,
-			     book_view, modify_contact_msgid,
+			     book_view, opid, modify_contact_msgid,
 			     modify_contact_search_handler, modify_contact_dtor);
 	}
 	else {
 		g_warning ("ldap_search_ext returned %d\n", ldap_error);
 		e_data_book_respond_modify (book,
-					 GNOME_Evolution_Addressbook_OtherError,
-					 NULL);
+					    opid,
+					    GNOME_Evolution_Addressbook_OtherError,
+					    NULL);
 		modify_contact_dtor ((LDAPOp*)modify_op);
 	}
 }
@@ -1639,8 +1676,9 @@ get_contact_handler (LDAPOp *op, LDAPMessage *res)
 		if (!e) {
 			g_warning ("uh, this shouldn't happen");
 			e_data_book_respond_get_contact (op->book,
-						      GNOME_Evolution_Addressbook_OtherError,
-						      "");
+							 op->opid,
+							 GNOME_Evolution_Addressbook_OtherError,
+							 "");
 			ldap_op_finished (op);
 			return;
 		}
@@ -1648,8 +1686,9 @@ get_contact_handler (LDAPOp *op, LDAPMessage *res)
 		contact = build_contact_from_entry (bl->priv->ldap, e, NULL);
 		vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
 		e_data_book_respond_get_contact (op->book,
-					      GNOME_Evolution_Addressbook_Success,
-					      vcard);
+						 op->opid,
+						 GNOME_Evolution_Addressbook_Success,
+						 vcard);
 		g_free (vcard);
 		g_object_unref (contact);
 		ldap_op_finished (op);
@@ -1666,13 +1705,18 @@ get_contact_handler (LDAPOp *op, LDAPMessage *res)
 		}
 		ldap_memfree (ldap_error_msg);
 
-		e_data_book_respond_get_contact (op->book, ldap_error_to_response (ldap_error), "");
+		e_data_book_respond_get_contact (op->book, 
+						 op->opid,
+						 ldap_error_to_response (ldap_error),
+						 "");
 		ldap_op_finished (op);
 	}
 	else {
 		g_warning ("unhandled result type %d returned", msg_type);
-		e_data_book_respond_get_contact (op->book, GNOME_Evolution_Addressbook_OtherError,
-					    "");
+		e_data_book_respond_get_contact (op->book,
+						 op->opid,
+						 GNOME_Evolution_Addressbook_OtherError,
+						 "");
 		ldap_op_finished (op);
 	}
 
@@ -1689,6 +1733,7 @@ get_contact_dtor (LDAPOp *op)
 static void
 e_book_backend_ldap_get_contact (EBookBackend *backend,
 				 EDataBook    *book,
+				 guint32       opid,
 				 const char   *id)
 {
 	LDAPGetContactOp *get_contact_op = g_new0 (LDAPGetContactOp, 1);
@@ -1711,13 +1756,14 @@ e_book_backend_ldap_get_contact (EBookBackend *backend,
 
 	if (ldap_error == LDAP_SUCCESS) {
 		ldap_op_add ((LDAPOp*)get_contact_op, backend, book,
-			     book_view, get_contact_msgid,
+			     book_view, opid, get_contact_msgid,
 			     get_contact_handler, get_contact_dtor);
 	}
 	else {
 		e_data_book_respond_get_contact (book,
-					    ldap_error_to_response (ldap_error),
-					    "");
+						 opid,
+						 ldap_error_to_response (ldap_error),
+						 "");
 		get_contact_dtor ((LDAPOp*)get_contact_op);
 	}
 }
@@ -1771,18 +1817,22 @@ contact_list_handler (LDAPOp *op, LDAPMessage *res)
 
 		if (ldap_error == LDAP_TIMELIMIT_EXCEEDED)
 			e_data_book_respond_get_contact_list (op->book,
+							      op->opid,
 							      GNOME_Evolution_Addressbook_SearchTimeLimitExceeded,
 							      contact_list_op->contacts);
 		else if (ldap_error == LDAP_SIZELIMIT_EXCEEDED)
 			e_data_book_respond_get_contact_list (op->book,
+							      op->opid,
 							      GNOME_Evolution_Addressbook_SearchSizeLimitExceeded,
 							      contact_list_op->contacts);
 		else if (ldap_error == LDAP_SUCCESS)
 			e_data_book_respond_get_contact_list (op->book,
+							      op->opid,
 							      GNOME_Evolution_Addressbook_Success,
 							      contact_list_op->contacts);
 		else
 			e_data_book_respond_get_contact_list (op->book,
+							      op->opid,
 							      GNOME_Evolution_Addressbook_OtherError,
 							      contact_list_op->contacts);
 
@@ -1791,6 +1841,7 @@ contact_list_handler (LDAPOp *op, LDAPMessage *res)
 	else {
 		g_warning ("unhandled search result type %d returned", msg_type);
 		e_data_book_respond_get_contact_list (op->book,
+						      op->opid,
 						      GNOME_Evolution_Addressbook_OtherError,
 						      NULL);
 		ldap_op_finished (op);
@@ -1810,7 +1861,8 @@ contact_list_dtor (LDAPOp *op)
 static void
 e_book_backend_ldap_get_contact_list (EBookBackend *backend,
 				      EDataBook    *book,
-				      const char *query)
+				      guint32       opid,
+				      const char   *query)
 {
 	LDAPGetContactListOp *contact_list_op = g_new0 (LDAPGetContactListOp, 1);
 	EBookBackendLDAP *bl = E_BOOK_BACKEND_LDAP (backend);
@@ -1840,11 +1892,12 @@ e_book_backend_ldap_get_contact_list (EBookBackend *backend,
 
 	if (ldap_error == LDAP_SUCCESS) {
 		ldap_op_add ((LDAPOp*)contact_list_op, backend, book,
-			     book_view, contact_list_msgid,
+			     book_view, opid, contact_list_msgid,
 			     contact_list_handler, contact_list_dtor);
 	}
 	else {
 		e_data_book_respond_get_contact_list (book,
+						      opid,
 						      ldap_error_to_response (ldap_error),
 						      NULL);
 		contact_list_dtor ((LDAPOp*)contact_list_op);
@@ -1853,10 +1906,11 @@ e_book_backend_ldap_get_contact_list (EBookBackend *backend,
 
 
 
-static EContactField email_ids[3] = {
+static EContactField email_ids[4] = {
 	E_CONTACT_EMAIL_1,
 	E_CONTACT_EMAIL_2,
-	E_CONTACT_EMAIL_3
+	E_CONTACT_EMAIL_3,
+	E_CONTACT_EMAIL_4
 };
 
 /* List property functions */
@@ -1864,18 +1918,18 @@ static void
 email_populate(EContact *contact, char **values)
 {
 	int i;
-	for (i = 0; values[i] && i < 3; i ++)
+	for (i = 0; values[i] && i < 4; i ++)
 		e_contact_set (contact, email_ids[i], values[i]);
 }
 
-struct berval**
+static struct berval**
 email_ber(EContact *contact)
 {
 	struct berval** result;
 	const char *emails[3];
 	int i, j, num = 0;
 
-	for (i = 0; i < 3; i ++) {
+	for (i = 0; i < 4; i ++) {
 		emails[i] = e_contact_get (contact, email_ids[i]);
 		if (emails[i])
 			num++;
@@ -1890,7 +1944,7 @@ email_ber(EContact *contact)
 		result[i] = g_new (struct berval, 1);
 
 	j = 0;
-	for (i = 0; i < 3; i ++) {
+	for (i = 0; i < 4; i ++) {
 		if (emails[i]) {
 			result[j]->bv_val = g_strdup (emails[i]);
 			result[j++]->bv_len = strlen (emails[i]);
@@ -1908,7 +1962,7 @@ email_compare (EContact *contact1, EContact *contact2)
 	const char *email1, *email2;
 	int i;
 
-	for (i = 0; i < 3; i ++) {
+	for (i = 0; i < 4; i ++) {
 		gboolean equal;
 		email1 = e_contact_get_const (contact1, email_ids[i]);
 		email2 = e_contact_get_const (contact2, email_ids[i]);
@@ -1935,7 +1989,7 @@ homephone_populate(EContact *contact, char **values)
 	}
 }
 
-struct berval**
+static struct berval**
 homephone_ber(EContact *contact)
 {
 	struct berval** result;
@@ -2003,7 +2057,7 @@ business_populate(EContact *contact, char **values)
 	}
 }
 
-struct berval**
+static struct berval**
 business_ber(EContact *contact)
 {
 	struct berval** result;
@@ -2070,7 +2124,7 @@ anniversary_populate (EContact *contact, char **values)
 	}
 }
 
-struct berval**
+static struct berval**
 anniversary_ber (EContact *contact)
 {
 	EContactDate *dt;
@@ -2123,7 +2177,7 @@ birthday_populate (EContact *contact, char **values)
 	}
 }
 
-struct berval**
+static struct berval**
 birthday_ber (EContact *contact)
 {
 	EContactDate *dt;
@@ -2180,25 +2234,33 @@ category_populate (EContact *contact, char **values)
 	g_list_free (categories);
 }
 
-struct berval**
+static struct berval**
 category_ber (EContact *contact)
 {
 	struct berval** result = NULL;
 	GList *categories;
-	int i;
+	const char *category_string;
+
+	category_string = e_contact_get (contact, E_CONTACT_CATEGORIES);
+	if (!category_string || !*category_string)
+		return NULL;
 
 	categories = e_contact_get (contact, E_CONTACT_CATEGORY_LIST);
 
 	if (g_list_length (categories) != 0) {
+		int i;
 		GList *iter;
 		result = g_new0 (struct berval*, g_list_length (categories) + 1);
 
-		for (iter = categories, i = 0; iter; iter = iter->next, i++) {
+		for (iter = categories, i = 0; iter; iter = iter->next) {
 			char *category = iter->data;
 
-			result[i] = g_new (struct berval, 1);
-			result[i]->bv_val = g_strdup (category);
-			result[i]->bv_len = strlen (category);
+			if (category && *category) {
+				result[i] = g_new (struct berval, 1);
+				result[i]->bv_val = g_strdup (category);
+				result[i]->bv_len = strlen (category);
+				i++;
+			}
 		}
 	}
 
@@ -2210,7 +2272,7 @@ category_ber (EContact *contact)
 static gboolean
 category_compare (EContact *contact1, EContact *contact2)
 {
-	char *categories1, *categories2;
+	const char *categories1, *categories2;
 	gboolean equal;
 
 	categories1 = e_contact_get_const (contact1, E_CONTACT_CATEGORIES);
@@ -2221,12 +2283,120 @@ category_compare (EContact *contact1, EContact *contact2)
 	else
 		equal = (categories1 == categories2);
 
-	g_free (categories1);
-	g_free (categories2);
+	return equal;
+}
+
+static void
+address_populate(EContact * card, char **values, EContactField field)
+{
+	if (values[0]) {
+		char *temp = g_strdup(values[0]);
+		char *i;
+		for (i = temp; *i != '\0'; i++) {
+			if (*i == '$') {
+				*i = '\n';
+			}
+		}
+		e_contact_set(card, field, temp);
+		g_free(temp);
+	}
+}
+
+static void
+home_address_populate(EContact * card, char **values)
+{
+	address_populate(card, values, E_CONTACT_ADDRESS_LABEL_HOME);
+}
+
+static void
+work_address_populate(EContact * card, char **values)
+{
+	address_populate(card, values, E_CONTACT_ADDRESS_LABEL_WORK);
+}
+
+static void
+other_address_populate(EContact * card, char **values)
+{
+	address_populate(card, values, E_CONTACT_ADDRESS_LABEL_OTHER);
+}
+
+static struct berval **
+address_ber(EContact * card, EContactField field)
+{
+	struct berval **result = NULL;
+	char *address, *i;
+
+	address = e_contact_get(card, field);
+	if (address) {
+		for (i = address; *i != '\0'; i++) {
+			if (*i == '\n') {
+				*i = '$';
+			}
+		}
+
+		result = g_new(struct berval *, 2);
+		result[0] = g_new(struct berval, 1);
+		result[0]->bv_val = address;
+		result[0]->bv_len = strlen(address);
+
+		result[1] = NULL;
+	}
+	return result;
+}
+
+static struct berval **
+home_address_ber(EContact * card)
+{
+	return address_ber(card, E_CONTACT_ADDRESS_LABEL_HOME);
+}
+
+static struct berval **
+work_address_ber(EContact * card)
+{
+	return address_ber(card, E_CONTACT_ADDRESS_LABEL_WORK);
+}
+
+static struct berval **
+other_address_ber(EContact * card)
+{
+	return address_ber(card, E_CONTACT_ADDRESS_LABEL_OTHER);
+}
+
+static gboolean
+address_compare(EContact * ecard1, EContact * ecard2, EContactField field)
+{
+	const char *address1, *address2;
+
+	gboolean equal;
+	address1 = e_contact_get_const(ecard1, field);
+	address2 = e_contact_get_const(ecard2, field);
+
+	if (address1 && address2)
+		equal = !strcmp(address1, address2);
+	else
+		equal = (!!address1 == !!address2);
 
 	return equal;
 }
 
+static gboolean
+home_address_compare(EContact * ecard1, EContact * ecard2)
+{
+	return address_compare(ecard1, ecard2, E_CONTACT_ADDRESS_LABEL_HOME);
+}
+
+static gboolean
+work_address_compare(EContact * ecard1, EContact * ecard2)
+{
+	return address_compare(ecard1, ecard2, E_CONTACT_ADDRESS_LABEL_WORK);
+}
+
+static gboolean
+other_address_compare(EContact * ecard1, EContact * ecard2)
+{
+	return address_compare(ecard1, ecard2, E_CONTACT_ADDRESS_LABEL_OTHER);
+}
+
 static void
 photo_populate (EContact *contact, struct berval **ber_values)
 {
@@ -2239,6 +2409,54 @@ photo_populate (EContact *contact, struct berval **ber_values)
         }
 }
 
+static struct berval **
+photo_ber (EContact *contact)
+{
+	struct berval **result = NULL;
+	EContactPhoto *photo;
+
+	photo = e_contact_get(contact, E_CONTACT_PHOTO);
+	if (photo) {
+
+		result = g_new(struct berval *, 2);
+		result[0] = g_new(struct berval, 1);
+		result[0]->bv_len = photo->length;
+		result[0]->bv_val = g_malloc (photo->length);
+		memcpy (result[0]->bv_val, photo->data, photo->length);
+		e_contact_photo_free (photo);
+
+		result[1] = NULL;
+	}
+
+	return result;
+}
+
+static gboolean
+photo_compare(EContact * ecard1, EContact * ecard2)
+{
+	EContactPhoto *photo1, *photo2;
+	gboolean equal;
+
+	photo1 = e_contact_get(ecard1, E_CONTACT_PHOTO);
+	photo2 = e_contact_get(ecard2, E_CONTACT_PHOTO);
+
+
+	if (photo1 && photo2) {
+		equal = ((photo1->length == photo2->length)
+			 && !memcmp (photo1->data, photo2->data, photo1->length));
+	}
+	else {
+		equal = (!!photo1 == !!photo2);
+	}
+
+	if (photo1)
+		e_contact_photo_free (photo1);
+	if (photo2)
+		e_contact_photo_free (photo2);
+
+	return equal;
+}
+
 static void
 cert_populate (EContact *contact, struct berval **ber_values)
 {
@@ -2844,8 +3062,6 @@ ldap_search_handler (LDAPOp *op, LDAPMessage *res)
 
 	d(printf ("ldap_search_handler (%p)\n", view));
 
-	bonobo_object_dup_ref(bonobo_object_corba_objref(BONOBO_OBJECT(view)), NULL);
-
 	if (!search_op->notified_receiving_results) {
 		search_op->notified_receiving_results = TRUE;
 		book_view_notify_status (op->view, _("Receiving LDAP search results..."));
@@ -2895,9 +3111,6 @@ ldap_search_handler (LDAPOp *op, LDAPMessage *res)
 		ldap_op_finished (op);
 
 	}
-
-
-	bonobo_object_release_unref(bonobo_object_corba_objref(BONOBO_OBJECT(view)), NULL);
 }
 
 static void
@@ -2905,22 +3118,20 @@ ldap_search_dtor (LDAPOp *op)
 {
 	LDAPSearchOp *search_op = (LDAPSearchOp*) op;
 
-	g_mutex_lock (e_data_book_view_get_mutex (search_op->view));
-
 	d(printf ("ldap_search_dtor (%p)\n", search_op->view));
 
 	/* unhook us from our EDataBookView */
 	g_object_set_data (G_OBJECT (search_op->view), "EBookBackendLDAP.BookView::search_op", NULL);
 
-	g_mutex_unlock (e_data_book_view_get_mutex (search_op->view));
+	bonobo_object_unref (search_op->view);
 
 	g_free (search_op);
 }
 
 static void
-e_book_backend_ldap_search (EBookBackendLDAP  	*bl,
-			    EDataBook         	*book,
-			    EDataBookView       *view)
+e_book_backend_ldap_search (EBookBackendLDAP *bl,
+			    EDataBook        *book,
+			    EDataBookView    *view)
 {
 	char *ldap_query;
 
@@ -2930,9 +3141,15 @@ e_book_backend_ldap_search (EBookBackendLDAP  	*bl,
 		LDAP *ldap = bl->priv->ldap;
 		int ldap_err;
 		int search_msgid;
-		
+		int limit;
+		int view_limit;
+
+		view_limit = e_data_book_view_get_max_results (view);
+		if (view_limit == -1 || view_limit > bl->priv->ldap_limit)
+			view_limit = bl->priv->ldap_limit;
+
 		printf ("searching server using filter: %s (expecting max %d results)\n", ldap_query,
-			e_data_book_view_get_max_results (view));
+			view_limit);
 
 		do {
 			book_view_notify_status (view, _("Searching..."));
@@ -2944,7 +3161,7 @@ e_book_backend_ldap_search (EBookBackendLDAP  	*bl,
 						    NULL, /* XXX */
 						    NULL, /* XXX */
 						    NULL, /* XXX timeout */
-						    e_data_book_view_get_max_results (view), &search_msgid);
+						    view_limit, &search_msgid);
 		} while (e_book_backend_ldap_reconnect (bl, view, ldap_err));
 
 		g_free (ldap_query);
@@ -2965,8 +3182,10 @@ e_book_backend_ldap_search (EBookBackendLDAP  	*bl,
 
 			op->view = view;
 
+			bonobo_object_ref (view);
+
 			ldap_op_add ((LDAPOp*)op, E_BOOK_BACKEND(bl), book, view,
-				     search_msgid,
+				     0, search_msgid,
 				     ldap_search_handler, ldap_search_dtor);
 
 			g_object_set_data (G_OBJECT (view), "EBookBackendLDAP.BookView::search_op", op);
@@ -2975,7 +3194,7 @@ e_book_backend_ldap_search (EBookBackendLDAP  	*bl,
 	}
 	else {
 		e_data_book_view_notify_complete (view,
-					       GNOME_Evolution_Addressbook_InvalidQuery);
+						  GNOME_Evolution_Addressbook_InvalidQuery);
 		return;
 	}
 
@@ -2986,30 +3205,15 @@ e_book_backend_ldap_start_book_view (EBookBackend  *backend,
 				     EDataBookView *view)
 {
 	EBookBackendLDAP *bl = E_BOOK_BACKEND_LDAP (backend);
-	LDAPSearchOp *op;
 
 	d(printf ("start_book_view (%p)\n", view));
 
-	g_mutex_lock (e_data_book_view_get_mutex (view));
+	/* we start at 1 so the user sees stuff as it appears and we
+	   aren't left waiting for more cards to show up, if the
+	   connection is slow. */
+	e_data_book_view_set_thresholds (view, 1, 3000);
 
-	op = g_object_get_data (G_OBJECT (view), "EBookBackendLDAP.BookView::search_op");
-
-	if (op) {
-		/* the only way op can be set here is if we raced with
-		   stop_book_view and lost.  make sure by checking
-		   op->aborted. */
-		if (!op->aborted) {
-			g_warning ("lost race with stop_book_view, but op->aborted != TRUE");
-		}
-
-		g_free (op);
-		g_object_set_data (G_OBJECT (view), "EBookBackendLDAP.BookView::search_op", NULL);
-	}
-	else {
-		e_book_backend_ldap_search (bl, NULL /* XXX ugh */, view);
-	}
-
-	g_mutex_unlock (e_data_book_view_get_mutex (view));
+	e_book_backend_ldap_search (bl, NULL /* XXX ugh */, view);
 }
 
 static void
@@ -3020,24 +3224,15 @@ e_book_backend_ldap_stop_book_view (EBookBackend  *backend,
 
 	d(printf ("stop_book_view (%p)\n", view));
 
-	g_mutex_lock (e_data_book_view_get_mutex (view));
-
 	op = g_object_get_data (G_OBJECT (view), "EBookBackendLDAP.BookView::search_op");
-	if (op) {
+	if (op)
 		ldap_op_finished ((LDAPOp*)op);
-	}
-	else {
-		op = g_new0 (LDAPSearchOp, 1);
-		op->aborted = TRUE;
-		g_object_set_data (G_OBJECT (view), "EBookBackendLDAP.BookView::search_op", op);
-	}
-
-	g_mutex_unlock (e_data_book_view_get_mutex (view));
 }
 
 static void
 e_book_backend_ldap_get_changes (EBookBackend *backend,
 				 EDataBook    *book,
+				 guint32       opid,
 				 const char   *change_id)
 {
 	/* FIXME: implement */
@@ -3049,6 +3244,7 @@ e_book_backend_ldap_get_changes (EBookBackend *backend,
 static void
 e_book_backend_ldap_authenticate_user (EBookBackend *backend,
 				       EDataBook    *book,
+				       guint32       opid,
 				       const char   *user,
 				       const char   *passwd,
 				       const char   *auth_method)
@@ -3083,7 +3279,8 @@ e_book_backend_ldap_authenticate_user (EBookBackend *backend,
 			}
 			else {
 				e_data_book_respond_authenticate_user (book,
-								    GNOME_Evolution_Addressbook_PermissionDenied);
+								       opid,
+								       GNOME_Evolution_Addressbook_PermissionDenied);
 				return;
 			}
 		}
@@ -3098,7 +3295,8 @@ e_book_backend_ldap_authenticate_user (EBookBackend *backend,
 						passwd);
 
 		e_data_book_respond_authenticate_user (book,
-						    ldap_error_to_response (ldap_error));
+						       opid,
+						       ldap_error_to_response (ldap_error));
 	}
 #ifdef ENABLE_SASL_BINDS
 	else if (!strncasecmp (auth_method, SASL_PREFIX, strlen (SASL_PREFIX))) {
@@ -3113,15 +3311,18 @@ e_book_backend_ldap_authenticate_user (EBookBackend *backend,
 
 		if (ldap_error == LDAP_NOT_SUPPORTED)
 			e_data_book_respond_authenticate_user (book,
-							    GNOME_Evolution_Addressbook_UnsupportedAuthenticationMethod);
+							       opid,
+							       GNOME_Evolution_Addressbook_UnsupportedAuthenticationMethod);
 		else
 			e_data_book_respond_authenticate_user (book,
-							    ldap_error_to_response (ldap_error));
+							       opid,
+							       ldap_error_to_response (ldap_error));
 	}
 #endif
 	else {
 		e_data_book_respond_authenticate_user (book,
-					    GNOME_Evolution_Addressbook_UnsupportedAuthenticationMethod);
+						       opid,
+						       GNOME_Evolution_Addressbook_UnsupportedAuthenticationMethod);
 		return;
 	}
 
@@ -3153,26 +3354,52 @@ e_book_backend_ldap_authenticate_user (EBookBackend *backend,
 
 static void
 e_book_backend_ldap_get_supported_fields (EBookBackend *backend,
-					  EDataBook    *book)
+					  EDataBook    *book,
+					  guint32       opid)
 
 {
 	EBookBackendLDAP *bl = E_BOOK_BACKEND_LDAP (backend);
 
 	e_data_book_respond_get_supported_fields (book,
-					       GNOME_Evolution_Addressbook_Success,
-					       bl->priv->supported_fields);
+						  opid,
+						  GNOME_Evolution_Addressbook_Success,
+						  bl->priv->supported_fields);
 }
 
 static void
 e_book_backend_ldap_get_supported_auth_methods (EBookBackend *backend,
-						EDataBook    *book)
+						EDataBook    *book,
+						guint32       opid)
 
 {
 	EBookBackendLDAP *bl = E_BOOK_BACKEND_LDAP (backend);
 
 	e_data_book_respond_get_supported_auth_methods (book,
-						     GNOME_Evolution_Addressbook_Success,
-						     bl->priv->supported_auth_methods);
+							opid,
+							GNOME_Evolution_Addressbook_Success,
+							bl->priv->supported_auth_methods);
+}
+
+static void
+ldap_cancel_op(void *key, void *value, void *data)
+{
+	EBookBackendLDAP *bl = data;
+	LDAPOp *op = value;
+
+	/* ignore errors, its only best effort? */
+	ldap_abandon_ext (bl->priv->ldap, op->id, NULL, NULL);
+}
+
+static GNOME_Evolution_Addressbook_CallStatus
+e_book_backend_ldap_cancel_operation (EBookBackend *backend, EDataBook *book)
+{
+	EBookBackendLDAP *bl = E_BOOK_BACKEND_LDAP (backend);
+
+	g_static_rec_mutex_lock (&bl->priv->op_hash_mutex);
+	g_hash_table_foreach (bl->priv->id_to_op, ldap_cancel_op, bl);
+	g_static_rec_mutex_unlock (&bl->priv->op_hash_mutex);
+
+	return GNOME_Evolution_Addressbook_Success;
 }
 
 static GNOME_Evolution_Addressbook_CallStatus
@@ -3234,6 +3461,15 @@ e_book_backend_ldap_load_source (EBookBackend             *backend,
 		return GNOME_Evolution_Addressbook_OtherError;
 }
 
+static void
+e_book_backend_ldap_remove (EBookBackend *backend, EDataBook *book, guint32 opid)
+{
+	/* if we ever add caching, we'll remove it here, but for now,
+	   just report back Success */
+
+	e_data_book_respond_remove (book, opid, GNOME_Evolution_Addressbook_Success);
+}
+
 static char*
 e_book_backend_ldap_get_static_capabilities (EBookBackend *backend)
 {
@@ -3295,6 +3531,9 @@ e_book_backend_ldap_dispose (GObject *object)
 		g_static_rec_mutex_unlock (&bl->priv->op_hash_mutex);
 		g_static_rec_mutex_free (&bl->priv->op_hash_mutex);
 
+		if (bl->priv->ldap)
+			ldap_unbind_ext_s (bl->priv->ldap, NULL, NULL);
+
 		if (bl->priv->poll_timeout != -1) {
 			printf ("removing timeout\n");
 			g_source_remove (bl->priv->poll_timeout);
@@ -3336,6 +3575,7 @@ e_book_backend_ldap_class_init (EBookBackendLDAPClass *klass)
 
 	/* Set the virtual methods. */
 	parent_class->load_source             = e_book_backend_ldap_load_source;
+	parent_class->remove                  = e_book_backend_ldap_remove;
 	parent_class->get_static_capabilities = e_book_backend_ldap_get_static_capabilities;
 
 	parent_class->create_contact          = e_book_backend_ldap_create_contact;
@@ -3349,6 +3589,7 @@ e_book_backend_ldap_class_init (EBookBackendLDAPClass *klass)
 	parent_class->authenticate_user       = e_book_backend_ldap_authenticate_user;
 	parent_class->get_supported_fields    = e_book_backend_ldap_get_supported_fields;
 	parent_class->get_supported_auth_methods = e_book_backend_ldap_get_supported_auth_methods;
+	parent_class->cancel_operation	      = e_book_backend_ldap_cancel_operation;
 
 	object_class->dispose = e_book_backend_ldap_dispose;
 }
